@@ -4,8 +4,7 @@
 
 Written by Pierre Quentel quentel.pierre@wanadoo.fr
 
-Published under the GPL licence - no warranty, used at user's risk etc. See
-http://www.gnu.org/copyleft/gpl.html
+Published under the BSD licence. See the file LICENCE.txt
 
 This script launches Karrigell with webservers.SimpleAsyncHTTPServer.Server as 
 web server
@@ -15,15 +14,15 @@ function)
 Requests are handled by class RequestHandler (one instance per request)
 """
 
-import webservers.SimpleAsyncHTTPServer as Server
-import KarrigellRequestHandler
-import k_config
-import k_utils
 import traceback
 import sys
 
-class asyncRequestHandler(Server.DialogManager,
-    KarrigellRequestHandler.KarrigellRequestHandler):
+import KarrigellRequestHandler
+from core import k_config, k_utils
+import webservers.SimpleAsyncHTTPServer as Server
+
+class asyncRequestHandler(KarrigellRequestHandler.KarrigellRequestHandler, 
+        Server.DialogManager):
     
     def handle_data(self):
         KarrigellRequestHandler.KarrigellRequestHandler.handle_data(self)
