@@ -3,60 +3,13 @@
 Written by Pierre Quentel quentel.pierre@wanadoo.fr
 
 Published under the BSD licence. See the file LICENCE.txt
-
-Request handler built upon SimpleHTTPRequestHandler, except for the following
-extensions :
-- .py : executes the Python file in the global namespace ; sys.stdout is
-redirected for writing on the HTTP output stream
-- .pih : Python Inside HTML (see the PythonInsideHTML module for syntax) :
-files mixing HTML and Python code, like PHP, ASP or JSP. The .pih file is
-first translated into Python code, then this script is run
-- .hip : HTML Inside Python : see the HIP module
-- .ks : Karrigell Service (a service with different pages in one Python script)
-
-Python programs run in a namespace including the following variables :
-- QUERY : an dictionnary made out of the Query string (GET requests) or
-from the request body (POST requests). The keys are the names of the form
-items and the values are the item's value, as a string or as a list if the
-field name ends with [] : for an <input type="text" name="info"> form item
-you get the value in QUERY["info"] as a string ; for
-<select multiple name="foo[]"> QUERY["foo"] will be a list
-- the same form field QUERY["info"] is also available with the
-shortcut _info (underscore + field name)
-- HEADERS : dictionnary with the HTTP headers sent by the client
-- RESPONSE : dictionnary with the HTTP headers to send in the response
-- SET_COOKIE : a SimpleCookie objet (in Python Lib's Cookie module) used in
-the response if set
-- AUTH_USER, AUTH_PASSWORD : user identifier and password submitted in the
-Authorization header if it's provided, None and None if not
-- ACCEPTED_LANGUAGES : a list of languages accepted by user (accept-language
-header field) ordered by preference
-- REQUEST_HANDLER : the current instance of KarrigellRequestHandler
-- THIS : the current instance of the Script class (see Template.py)
-
-Python programs can also raise exceptions handled by the Karrigell
-engine :
-- SCRIPT_END : to terminate a script (added for readability reasons in the
-.pih and .hip scripts, see examples)
-- HTTP_REDIRECTION : raise HTTP_REDIRECTION,url causes the server to redirect
-the client towards the given url
-
-Session management by cookies is supported :
-- in a script, a call to the Session() function returns a session object :
-sessionObject=Session()
-- you can then set and read attributes of this object :
-    sessionObject.name=name
-    name=sessionObject.name
-- to end the session : sessionObject.close() erases the cookie
-
-A maximum of 1000 simultaneous sessions is supported
 """
 
-import sys, os, string, cStringIO, traceback, base64, time, gettext, copy
+import sys, os, string, cStringIO, traceback, base64, time, copy
 import Cookie, urlparse, mimetypes, BaseHTTPServer, cgi, urllib
 import logging.handlers
 
-# Karrigell-sepcific modules
+# Karrigell-specific modules
 from core import k_config
 import Template, URLResolution, k_utils, k_session, k_script, modify_request
 import debugger.k_debugger
@@ -64,7 +17,7 @@ from k_encodings import k_encoding_charsets, k_encoding
 from k_stringio import KStringIO as StringIO
 from LocalizedRequestHandler import LocalizedRequestHandler
 
-__version__ = "2.2.5"
+__version__ = "2.3.2"
 
 os.chdir(k_config.serverDir)
 
@@ -84,8 +37,10 @@ mimetypes.types_map.update({
 mimetypes.types_map.update(k_config.extensions_map)
 
 # for imports
-if not os.getcwd() in sys.path:
-    sys.path.append(os.getcwd())
+dirs = [ os.getcwd(),os.path.join(os.getcwd(),'buzhug') ]
+for dir_ in dirs:
+    if not dir_ in sys.path:
+        sys.path.append(dir_)
 
 # set up logging
 if k_config.loggingFile:
@@ -234,6 +189,88 @@ class KarrigellRequestHandler(LocalizedRequestHandler):
             "os":os,"Cookie":Cookie,"string":string})
             # frequently used modules needn't be imported in scripts
 
+        d,depth = k_utils.pathInDirs(self.path_without_qs,self.cgi_directories)
+        self.cgi_info = None
+        if d:
+            self.cgi_info = d,self.path[len(d)+1:]
+            self.make_cgi_env()
+    
+    def make_cgi_env(self):
+        # set environment for cgi scripts
+        # copied from CGIHTTPServer
+        dir_, rest = self.cgi_info
+        i = rest.rfind('?')
+        if i >= 0:
+            rest, query = rest[:i], rest[i+1:]
+        else:
+            query = ''
+        i = rest.find('/')
+        if i >= 0:
+            script, rest = rest[:i], rest[i:]
+        else:
+            script, rest = rest, ''
+        scriptname = dir_ + '/' + script
+        env = {}
+        env['SERVER_SOFTWARE'] = self.version_string()
+        env['SERVER_NAME'] = self.server.server_name
+        env['GATEWAY_INTERFACE'] = 'CGI/1.1'
+        env['SERVER_PROTOCOL'] = self.protocol_version
+        env['SERVER_PORT'] = str(self.server.server_port)
+        env['REQUEST_METHOD'] = self.command
+        uqrest = urllib.unquote(rest)
+        env['PATH_INFO'] = uqrest
+        env['PATH_TRANSLATED'] = self.translate_path(uqrest)
+        env['SCRIPT_NAME'] = scriptname
+        if query:
+            env['QUERY_STRING'] = query
+        host = self.address_string()
+        if host != self.client_address[0]:
+            env['REMOTE_HOST'] = host
+        env['REMOTE_ADDR'] = self.client_address[0]
+        authorization = self.headers.getheader("authorization")
+        if authorization:
+            authorization = authorization.split()
+            if len(authorization) == 2:
+                import base64, binascii
+                env['AUTH_TYPE'] = authorization[0]
+                if authorization[0].lower() == "basic":
+                    try:
+                        authorization = base64.decodestring(authorization[1])
+                    except binascii.Error:
+                        pass
+                    else:
+                        authorization = authorization.split(':')
+                        if len(authorization) == 2:
+                            env['REMOTE_USER'] = authorization[0]
+        # XXX REMOTE_IDENT
+        if self.headers.typeheader is None:
+            env['CONTENT_TYPE'] = self.headers.type
+        else:
+            env['CONTENT_TYPE'] = self.headers.typeheader
+        length = self.headers.getheader('content-length')
+        if length:
+            env['CONTENT_LENGTH'] = length
+        accept = []
+        for line in self.headers.getallmatchingheaders('accept'):
+            if line[:1] in "\t\n\r ":
+                accept.append(line.strip())
+            else:
+                accept = accept + line[7:].split(',')
+        env['HTTP_ACCEPT'] = ','.join(accept)
+        ua = self.headers.getheader('user-agent')
+        if ua:
+            env['HTTP_USER_AGENT'] = ua
+        co = filter(None, self.headers.getheaders('cookie'))
+        if co:
+            env['HTTP_COOKIE'] = ', '.join(co)
+        # XXX Other HTTP_* headers
+        # Since we're setting the env in the parent, provide empty
+        # values to override previously set values
+        for k in ('QUERY_STRING', 'REMOTE_HOST', 'CONTENT_LENGTH',
+                  'HTTP_USER_AGENT', 'HTTP_COOKIE'):
+            env.setdefault(k, "")
+        os.environ.update(env)
+
     def handle_data(self):
         """Wrap in a try/except clause for uncaught exceptions 
         to prevent crashing the server"""
@@ -303,6 +340,9 @@ class KarrigellRequestHandler(LocalizedRequestHandler):
         if k_config.loggingFile:
             logging.info(msg)
 
+    def translate_path(self,path):
+        return URLResolution.translate_path(path)
+
     def try_handle_data(self):
         """Handle a GET or POST request"""
         sys.stdout = k_utils.Stdout()
@@ -328,8 +368,10 @@ class KarrigellRequestHandler(LocalizedRequestHandler):
             self.host = 0
         for k in k_config.virtual_hosts[self.host].keys():
             setattr(k_config,k,k_config.virtual_hosts[self.host][k])
+        URLResolution.k_config = k_config
 
         fileName=URLResolution.translate_path(self.path_without_qs)
+        
         subpath = URLResolution.getScriptElements(self.path_without_qs)
         self.COOKIE = Cookie.SimpleCookie()
         self.SET_COOKIE=Cookie.SimpleCookie()
@@ -352,6 +394,7 @@ class KarrigellRequestHandler(LocalizedRequestHandler):
                 fileName+=ext
                 self.path_without_qs+=ext
             except IOError:
+                self.install_translations()
                 self.send_error(404,_("No file matching url ")+self.path_without_qs)
                 return
             except URLResolution.DuplicateExtensionError,msg:
@@ -432,6 +475,11 @@ class KarrigellRequestHandler(LocalizedRequestHandler):
 
         os.chdir(os.path.dirname(fileName))
 
+        #if self.command == 'HEAD':
+        #    """HEAD request : stop here"""
+        #    self.karrigellSendResponse(200,"Ok")
+        #    return
+
         script = None
         # cache
         if self.cache(fileName):
@@ -462,7 +510,7 @@ class KarrigellRequestHandler(LocalizedRequestHandler):
                 self.outputStream.write(out)
                 self.karrigellSendResponse(200,'Ok')
                 return
-        
+
         if script:
             self.prepare_env()
             # add attributes to the script object
@@ -496,7 +544,8 @@ class KarrigellRequestHandler(LocalizedRequestHandler):
                 for item in self.RESPONSE.keys():
                     self.send_header(item,self.RESPONSE[item])
                 self.end_headers()
-                self.copyfile(f, self.wfile)
+                if not self.command == "HEAD":
+                    self.copyfile(f, self.wfile)
 
         # in any case, restore current directory
         os.chdir(saveDir)
@@ -560,13 +609,6 @@ class KarrigellRequestHandler(LocalizedRequestHandler):
             for m in self.imported[self.host].keys():
                 sys.modules[m] = self.imported[self.host][m][0]
 
-        # for security reasons, hide some modules
-        # for virtual hosts, prevents scripts to access session information
-        save_modules = {}
-        for m in ['k_session','KarrigellRequestHandler']:
-            save_modules[m] = sys.modules[m]
-            del sys.modules[m]
-
         # self.execution is used in Include()
         self.execution=Template.ExecContext(script,self.nameSpace,
             self.path, self)
@@ -574,8 +616,20 @@ class KarrigellRequestHandler(LocalizedRequestHandler):
         # delete the imported modules whose source code has changed
         self.reload_modules()
 
+        # for security reasons, hide some modules
+        # for virtual hosts, prevents scripts to access session information
+        save_modules = {}
+        for m in ['k_session','KarrigellRequestHandler']:
+            save_modules[m] = sys.modules[m]
+            del sys.modules[m]
+
         try:
-            self.outputStream.write(self.execution())
+            try:
+                self.outputStream.write(self.execution())
+            finally:
+                # restore saved modules
+                for m in save_modules.keys():
+                    sys.modules[m] = save_modules[m]
         # catches defined exceptions
         except k_script.HTTP_REDIRECTION,url:    
             # HTTP redirection towards a given URL
@@ -588,11 +642,13 @@ class KarrigellRequestHandler(LocalizedRequestHandler):
             pass
         else:
             # if everything's ok, send response
-            self.karrigellSendResponse(200,"Ok")
-
-        # restore saved modules
-        for m in save_modules.keys():
-            sys.modules[m] = save_modules[m]
+            if not self.cgi_info:
+                self.karrigellSendResponse(200,"Ok")
+            else:
+                self.send_response(200,"Ok")
+                self.outputStream.seek(0)
+                if not self.command == "HEAD":
+                    self.copyfile(self.outputStream, self.wfile)
         
         # save session object
         k_session.store(self.sessionId,self.sessionObject)
@@ -732,6 +788,8 @@ class KarrigellRequestHandler(LocalizedRequestHandler):
     def send_error(self, code, message=None):
         """Overrides BaseHTTPServer.BaseHTTPRequestHandler's send_error with
         additional output : server version and date/time"""
+        self.install_translations()
+        self.translateClassAttributes()
         try:
             short, long = self.responses[code]
         except KeyError:
@@ -743,17 +801,6 @@ class KarrigellRequestHandler(LocalizedRequestHandler):
         self.send_response(code, message)
         self.send_header("Content-Type", content_type())
         self.end_headers()
-        # translation 
-        gettext._translations={}
-        self.get_language()
-        try: 
-            self.t=gettext.translation("messages", 
-            os.path.join(k_config.serverDir,"translations"), 
-            self.accepted_languages) 
-        except: 
-            self.t=gettext.NullTranslations() 
-        self.t.install(unicode=1) 
-        self.translateClassAttributes()
         err_msg = (self.error_message_format %
                          {'code': code,
                           'message': message,
@@ -762,6 +809,10 @@ class KarrigellRequestHandler(LocalizedRequestHandler):
                           'time':self.date_time_string()})
         self.wfile.write(k_encoding.encode(err_msg, k_config.outputEncoding))
 
+    def address_string(self):
+        """Overrides address_string to remove hostname resolution"""
+        return self.client_address[0] 
+    
     def testGzip(self):
         """Test if content should be gzipped"""
         if not k_config.gzip:
@@ -812,14 +863,10 @@ class KarrigellRequestHandler(LocalizedRequestHandler):
         for item in self.RESPONSE.keys():
             self.send_header(item,self.RESPONSE[item])
         self.end_headers()
-        # output stream is written on the socket fileobject
-        self.outputStream.seek(0)
-        self.copyfile(self.outputStream, self.wfile)
-
-
-# for internationalization
-t=gettext.NullTranslations()
-t.install(unicode=1) # unicode=1 is not needed here...
+        if not self.command == "HEAD":
+            # output stream is written on the socket fileobject
+            self.outputStream.seek(0)
+            self.copyfile(self.outputStream, self.wfile)
 
 # Python may not have gzip support
 gzip_support = False
