@@ -1,14 +1,15 @@
 import os
-import kirbybase
-db = kirbybase.KirbyBase()
+import buzhug
+from datetime import datetime
+db = buzhug.Base('entries')
+
+try:
+    db.open()
+except IOError:
+    db.create(('parent',int),('thread',int),('author',str),
+        ('title',str),('content',str),('date',datetime),
+        ('lastDate',datetime),('numChildren',int))
 
-def get_time_format(languages):
-    """Return the formatting string for time, according to language preference"""
-    if languages and languages[0].lower().startswith('fr'):
-        return '%d/%m/%Y %H:%M'
-    else:
-        return '%Y-%m-%d %H:%M'
-        
 class Message:
 
     def __init__(self,author,title,content,parent=-1,thread=-1,date=None, lastDate=None):
