@@ -7,6 +7,6 @@ import datetime
 from portalDb import db
 
 so=Session()
-db.insert('news.tbl',[so.login,_newsTitle,_newsBody,datetime.datetime.today()])
+db['news'].insert(so.login,_newsTitle,_newsBody,datetime.datetime.today())
 
 raise HTTP_REDIRECTION,"index.pih"
