@@ -2,5 +2,5 @@
 
 from portalDb import db
 so=Session()
-db.update('user.tbl',['recno'],[so.user],{'bgcolor':_bgcolor,'fontfamily':_fontfamily})
+db['users'].update(db['users'][so.user],bgcolor=_bgcolor,fontfamily=_fontfamily)
 raise HTTP_REDIRECTION,"index.pih"
