@@ -6,11 +6,9 @@ if len(_newsBody)>255:
 
 import datetime
 from portalDb import db
-# retrieve object
-news=db.select('news.tbl',['recno'],[int(_id)],returnType='object')[0]
 
 # update values
-db.update('news.tbl',['recno'],[int(_id)],
-    {'title':_newsTitle, 'body':_newsBody, 'date':datetime.datetime.today()})
+db['news'].update(db['news'][int(_id)],
+    title=_newsTitle, body=_newsBody, date=datetime.datetime.today())
     
 raise HTTP_REDIRECTION,"index.pih"
