@@ -60,7 +60,7 @@ import cStringIO
 import socket
 import select
 import cgi
-from SimpleHTTPServer import SimpleHTTPRequestHandler
+from CGIHTTPServer import CGIHTTPRequestHandler
 import tempfile
 
 class CloseRequest(Exception):
@@ -101,13 +101,13 @@ class wfile:
     def __len__(self):
         return len("".join(self.buff))
 
-class DialogManager(SimpleHTTPRequestHandler):
+class DialogManager(CGIHTTPRequestHandler):
 
     blocksize = 131072  # size of blocks sent to the socket
     bufsize = 131072    # size of in-memory buffer
-    server_version = "SimpleAsyncHTTP/1.0"
 
-    def __init__(self, request, client_address):
+    def __init__(self, server, request, client_address):
+        self.server = server
         self.request, self.client_address = request, client_address
         self.request.setblocking(0)
         self.incoming = cStringIO.StringIO() # receives the HTTP request
@@ -121,6 +121,12 @@ class DialogManager(SimpleHTTPRequestHandler):
         self.body = {}
         self.pre_handle_data()
 
+    def do_HEAD(self):
+        """Serve a HEAD request - same as GET, treated differently in
+        handle_data()"""
+        self.do_GET()
+        return
+
     def do_POST(self):
         """Prepare to read the request body"""
         self.cont_length = int(self.headers.getheader('content-length') or 0)
@@ -140,6 +146,8 @@ class DialogManager(SimpleHTTPRequestHandler):
             self.pre_handle_data()
 
     def pre_handle_data(self):
+        self.incoming.seek(0)
+        sys.stdin = self.incoming # compatibility with cgi
         if len(self.wfile):
             self.sources.append(wfile())
             self.wfile = self.sources[-1]
@@ -167,6 +175,8 @@ class DialogManager(SimpleHTTPRequestHandler):
             return
         length = os.fstat(f.fileno())[6]
         self.send_response(200)
+        if self.command == "HEAD":
+            return
         self.send_header("Content-type", ctype)
         self.send_header("Content-Length", str(length))
         self.end_headers()
@@ -253,11 +263,13 @@ class DialogManager(SimpleHTTPRequestHandler):
 class Server:
 
     def __init__(self, server_address, RequestHandlerClass):
+        self.server_name = "SimpleAsyncHTTP/1.0"
+        self.server_port = server_address[1]
         self.RequestHandlerClass = RequestHandlerClass
         self.socket = socket.socket(socket.AF_INET,socket.SOCK_STREAM)
         self.socket.setblocking(0)
         self.socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
-        # new exception treatment to avoid some runtime errors if apache
+        # exception treatment to avoid some runtime errors if apache
         # or another http server is running. Author: Marcelo Santos Araujo
         try:
             self.socket.bind(server_address)
@@ -273,8 +285,8 @@ class Server:
             request, client_address = self.socket.accept()
         except socket.error:
             return
-        self.client_handlers[request] = self.RequestHandlerClass(request, 
-            client_address)
+        self.client_handlers[request] = self.RequestHandlerClass(self,
+            request, client_address)
 
     def loop(self):
         while True:
